const { getClient, checkParams, getResponse } = require('./utils.js');
var _ = require('underscore');

var es_host = process.argv[2];
var es_port = process.argv[3];
var cbd_env = process.argv[4];
var start_time = process.argv[5] || new Date().getTime() - 30*24*60*60*1000;
var recent_start_time = process.argv[6] || new Date().getTime() - 3*24*60*60*1000;
var last_day_end_time = new Date().getTime();
var last_day_start_time = last_day_end_time - 24*60*60*1000;
var es_user = process.env.ES_USER;
var es_password = process.env.ES_PASSWORD;
var batch = 200;
var max_org_id = 1;

// Comment this out until last day definition is clarified with the PMs
// For consumption tracking calculations
// var start_time_last_day = new Date();
// var end_time_last_day = new Date();

// end_time_last_day.setUTCHours(0,0,0,0);
// start_time_last_day.setUTCHours(0,0,1,0);
// start_time_last_day.setUTCHours(end_time_last_day.getUTCHours() - 24);

/*
class SensorStats {
    constructor() {
        this.total = 0;
        this.totalActive = 0;
        this.activeWindows = 0;
        this.inactiveWindows = 0;
        this.activeMac = 0;
        this.inactiveMac = 0;
        this.countByOS = [];
        this.countBySensorVersion = [];
    }
};

class Organization {
        constructor(name) {
            this.env = cbd_env;
            this.name = name;
            this.registered = new SensorStats();
            this.deregistered = 0;
            this.bypass = 0;
            this.quarantine = 0;
            this.pending = 0;
        }
};
*/

function getQuery (start_org_id, end_org_id, is_mcs2) {
  var query = {
      ...(!is_mcs2 && {requestTimeout: 30000}),
      index: 'a_devicestats',
      type: 'device',
      body: {
          "query": {
              "bool": {
                  "must": {
                      "query_string": {
                          "query": "*"
                      }
                  },
                  "filter" : {
                      "range": {
                          "organizationId": {
                              "gte": + start_org_id,
                              "lt" : + end_org_id
                          }
                      }
                  }
              }
          },
          "aggregations" : {
              "orgIds" : {
                "terms": {
                  "field": "organizationId",
                  "size": 10000
                },
                "aggregations": {
                  "orgs": {
                    "terms": {
                      "field": "organizationName",
                      "size": 100
                    }
                  },
                  "status": {
                    "terms": {
                      "field": "status.full_status",
                      "size": 10
                    },
                    "aggregations": {
                      "deviceType": {
                        "terms": {
                          "field": "deviceType.full_deviceType",
                          "size": 10
                        },
                        "aggregations": {
                          "activeDevices": {
                            "filter": {
                              "range": {
                                "lastContact": {
                                  "gte": +start_time
                                }
                              }
                            }
                          },
                          "activeDevicesRecent": {
                              "filter": {
                                "range": {
                                  "lastContact": {
                                    "gte": +recent_start_time
                                  }
                                }
                              }
                          },
                          "inactiveDevices": {
                            "filter": {
                              "range": {
                                "lastContact": {
                                  "lt": +start_time
                                }
                              }
                            }
                          },
                          "inactiveDevicesRecent": {
                            "filter": {
                              "range": {
                                "lastContact": {
                                  "lt": +recent_start_time
                                }
                              }
                            }
                          },
                          "countByOSAndSensorVersion": {
                            "terms": {
                              "field": "osVersion.full_osVersion",
                              "size": 100
                              },
                              "aggregations": {
                                "countBySensorVersion": {
                                  "terms": {
                                    "field": "sensorVersion.full_sensorVersion",
                                    "size": 100
                                  }
                                }
                              }
                            },
                            "activeSensorData": {
                              "filter": {
                                  "range": {
                                      "lastContact": {
                                          "gte": +start_time
                                      }
                                  }
                              },
                              "aggregations": {
                                  "countByOSAndSensorVersion": {
                                      "terms": {
                                          "field": "osVersion.full_osVersion",
                                          "size": 100
                                      },
                                      "aggregations": {
                                          "countBySensorVersion": {
                                          "terms": {
                                              "field": "sensorVersion.full_sensorVersion",
                                              "size": 100
                                              }
                                          }
                                      }
                                  },
                                  "countByDeploymentType": {
                                    "terms": {
                                      "field": "deploymentType.full_deploymentType",
                                      "size": 100
                                    },
                                    "aggregations": {
                                      "countBySubDeploymentType": {
                                        "terms": {
                                          "field": "subDeploymentType",
                                          "size": 100
                                        },
                                        "aggregations": {
                                          "goldenCount": {
                                              "filter": {
                                                  "term": {
                                                      "isGoldenDevice": true
                                                  }
                                              }
                                          }
                                        }
                                      }
                                    }
                                  }
                              }
                          },
                          "activeSensorDataRecent": {
                              "filter": {
                                  "range": {
                                      "lastContact": {
                                          "gte": +recent_start_time
                                      }
                                  }
                              },
                              "aggregations": {
                                  "countByOSAndSensorVersion": {
                                      "terms": {
                                          "field": "osVersion.full_osVersion",
                                          "size": 100
                                      },
                                      "aggregations": {
                                          "countBySensorVersion": {
                                          "terms": {
                                              "field": "sensorVersion.full_sensorVersion",
                                              "size": 100
                                              }
                                          }
                                      }
                                  },
                                  "countByDeploymentType": {
                                    "terms": {
                                      "field": "deploymentType.full_deploymentType",
                                      "size": 100
                                    },
                                    "aggregations": {
                                      "countBySubDeploymentType": {
                                        "terms": {
                                          "field": "subDeploymentType",
                                          "size": 100
                                        },
                                        "aggregations": {
                                          "goldenCount": {
                                            "filter": {
                                              "term": {
                                                "isGoldenDevice": true
                                              }
                                            }
                                          }
                                        }
                                      }
                                    }
                                  }
                              }
                          }
                        }
                      },
                      "bypassTotalConsumedLastDay": {
                        "filter": {
                          "range": {
                            "lastContact": {
                              "gte": +last_day_start_time
                            }
                          }
                        }
                      }
                    }
                  },
                  "totalConsumedLastDay": {
                    "filter": {
                      "range": {
                        "lastContact": {
                          "gte": +last_day_start_time
                        }
                      }
                    },
                    "aggregations": {
                      "totalConsumedByDeploymentTypeLastDay": {
                        "terms": {
                          "field": "deploymentType.full_deploymentType",
                          "size": 100
                        },
                        "aggregations": {
                          "totalConsumedLastDayBySubDeploymentType": {
                            "terms": {
                              "field": "subDeploymentType",
                              "size": 100
                            },
                            "aggregations": {
                              "goldenCount": {
                                  "filter": {
                                    "term": {
                                    "isGoldenDevice": true
                                  }
                                }
                              }
                            }
                          }
                        }
                      },
                      "countByOSAndSensorVersion": {
                        "terms": {
                          "field": "osVersion.full_osVersion",
                          "size": 100
                        },
                        "aggregations": {
                          "countBySensorVersion": {
                            "terms": {
                              "field": "sensorVersion.full_sensorVersion",
                              "size": 100
                            }
                          }
                        }
                      }
                    }
                  },
                  "protectionHoursByDeploymentTypeLastDay": {
                    "terms": {
                      "field": "deploymentType.full_deploymentType"
                    },
                    "aggregations": {
                      "protectionTime": {
                        "sum": {
                          "script": {
                            "source": "if (doc['registeredTime'].value != 0) { if (doc['deregisteredTime'].value != 0) {if(doc['deregisteredTime'].value > params['start_time']){return (Math.min(doc['deregisteredTime'].value, params['end_time']) - Math.max(doc['registeredTime'].value, params['start_time']))/1000/60/60}} else {return (params['end_time'] - Math.max(doc['registeredTime'].value, params['start_time']))/1000/60/60}}",
                            "params": {
                              "end_time": +last_day_end_time,
                              "start_time": +last_day_start_time
                            }
                          }
                        }
                      },
                      "registeredCountLastDay": {
                        "filter": {
                          // "range": {
                          //   "registeredTime": {
                          //     "gte": +start_time_last_day,
                          //     "lte": +end_time_last_day
                          //   }
                          // }
                          "range": {
                            "registeredTime": {
                              "gte": +last_day_start_time
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
          },
            "size": 0
          }
      };
      //console.log("query: " + JSON.stringify(query));
      return query;
}

function processResponse(response, is_mcs2) {
  agg = getResponse(response, is_mcs2);

  if (!_.isUndefined(agg.aggregations && !_.isUndefined(agg.aggregations.orgIds) && !_.isUndefined(agg.aggregations.orgIds.buckets))) {
      var org_buckets = agg.aggregations.orgIds.buckets;
      // console.log("processing stats for (" + org_buckets.length + ") orgs");
      org_buckets.forEach(function (orgIdEntry) {
          var org = {
                      "id": orgIdEntry.key,
                      "name": "",
                      "env": cbd_env,
                      "registered" : {
                              "total": 0,
                              "totalActive": 0,
                              "activeWindows": 0,
                              "inactiveWindows": 0,
                              "inactiveMac": 0,
                              "inactiveLinux": 0,
                              "activeMac": 0,
                              "activeLinux": 0,
                              "countByOS": [],
                              "countBySensorVersion": [],
                              "countByOSAndSensorVersion": [],
                              "totalRecent": 0,
                              "totalActiveRecent": 0,
                              "activeWindowsRecent": 0,
                              "activeMacRecent": 0,
                              "activeLinuxRecent": 0,
                              "inactiveWindowsRecent": 0,
                              "inactiveMacRecent": 0,
                              "inactiveLinuxRecent": 0,
                              "activeCountByOS": [],
                              "activeCountBySensorVersion": [],
                              "activeCountByOSAndSensorVersion": [],
                              "activeCountByOSRecent": [],
                              "activeCountBySensorVersionRecent": [],
                              "activeCountByOSAndSensorVersionRecent": [],
                              "activeCountByDeploymentType": [],
                              "activeCountByDeploymentTypeRecent": [],
                              "activeGoldenDevices": 0,
                              "activeGoldenDevicesRecent": 0
                      },
                      "deregistered": 0,
                      "bypass": 0,
                      "pending": 0,
                      "quarantine": 0,
                      "deleted": 0,
                      "activeBypass": {
                        "totalActive": 0,
                        "totalActiveRecent": 0
                      },
                      "totalConsumedLastDay": 0,
                      "totalConsumedByDeploymentTypeLastDay": [],
                      "totalConsumedByOsLastDay": [],
                      "totalConsumedBySensorVersionLastDay": [],
                      "totalConsumedByOsAndSensorVersionLastDay": [],
                      "totalConsumedGoldenDevicesLastDay": 0,
                      "bypassTotalConsumedLastDay": 0,
                      "protectionHoursByDeploymentTypeLastDay": [],
                      "registeredCountByDeploymentTypeLastDay": []
                  };

      function populateCountByOs(osAndSensorInfo, osExportInfo) {
        osExportInfo.push({"key": osAndSensorInfo.key, "count": osAndSensorInfo.doc_count});
      }

      function populateCountBySensorVersion(sensorInfo, sensorExportInfo) {
        if(sensorExportInfo.key == sensorInfo.key) {
            var index = sensorExportInfo.findIndex(e => e.key == sensorInfo.key);
            sensorExportInfo[index].count = sensorExportInfo[index].count + sensorInfo.doc_count;
        } else {
          sensorExportInfo.push({
                "key": sensorInfo.key,
                "count": sensorInfo.doc_count
            })
        }
      }

      function populateCountByOSAndSensorVersion(osAndSensorInfo, sensorInfo, osAndSensorExportInfo) {
          osAndSensorExportInfo.push({
              "os": osAndSensorInfo.key,
              "sensorVersion": sensorInfo.key,
              "count": sensorInfo.doc_count
          });
      }

      function populateAllCountBy(osAndSensorInfo, osExportInfo, sensorExportInfo, osAndSensorExportInfo) {
        populateCountByOs(osAndSensorInfo, osExportInfo);

        osAndSensorInfo.countBySensorVersion.buckets.forEach(sensorInfo => {
            populateCountByOSAndSensorVersion(osAndSensorInfo, sensorInfo, osAndSensorExportInfo);
            populateCountBySensorVersion(sensorInfo, sensorExportInfo);
        })
      }

      function populateGoldenCount(subDeploymentTypeBuckets, goldenDevicesExportInfo) {
        goldenDevicesExportInfo = subDeploymentTypeBuckets
          .map(subDeploymentTypeSensorInfo => subDeploymentTypeSensorInfo.goldenCount.doc_count)
          .reduce((a, b) => a + b, 0);
      }

      function populateDeploymentType(deploymentTypeSensorInfo, subDeploymentTypeSensorInfo, deploymentTypeExportInfo) {
          var deploymentType = _.findWhere(deploymentTypeExportInfo, {"deploymentType" : deploymentTypeSensorInfo.key});

          if(deploymentType !== undefined) {
            deploymentType.count = deploymentType.count + deploymentTypeSensorInfo.doc_count;
          } else {
            deploymentTypeExportInfo.push({
              "deploymentType": deploymentTypeSensorInfo.key,
              "count": deploymentTypeSensorInfo.doc_count,
              "subDeploymentType": []
            })
          }

          subDeploymentTypeSensorInfo.forEach(subDeploymentTypeSensorInfoInner => populateSubDeploymentType(deploymentTypeSensorInfo, deploymentTypeExportInfo, subDeploymentTypeSensorInfoInner, deploymentTypeSensorInfo.key));
      }

      function populateSubDeploymentType(deploymentTypeSensorInfo, deploymentTypeExportInfo, subDeploymentTypeSensorInfo, deploymentTypePropertyName) {
        var deploymentType = _.findWhere(deploymentTypeExportInfo, {"deploymentType" : deploymentTypePropertyName});

        if(deploymentType !== undefined)
          var subDeploymentType = _.findWhere(deploymentType.subDeploymentType, {"type": subDeploymentTypeSensorInfo.key});

        if(subDeploymentType !== undefined) {
          subDeploymentType.count = subDeploymentType.count + subDeploymentTypeSensorInfo.doc_count;
        } else {
          deploymentType.subDeploymentType.push({
              "type": subDeploymentTypeSensorInfo.key,
              "count": subDeploymentTypeSensorInfo.doc_count
            });
        }
      }

      function populateCountFields(osBucket, osName) {
          org.registered['active' + osName] = osBucket.activeDevices.doc_count;
          org.registered['active' + osName + 'Recent'] = osBucket.activeDevicesRecent.doc_count;
          org.registered['inactive' + osName] = osBucket.inactiveDevices.doc_count;
          org.registered['inactive' + osName + 'Recent'] = osBucket.inactiveDevicesRecent.doc_count;

          osBucket.countByOSAndSensorVersion.buckets.forEach(osAndSensorInfo =>
            populateAllCountBy(
              osAndSensorInfo,
              org.registered["countByOS"],
              org.registered["countBySensorVersion"],
              org.registered["countByOSAndSensorVersion"]));

          osBucket.activeSensorData.countByOSAndSensorVersion.buckets.forEach(osAndSensorInfo =>
            populateAllCountBy(
              osAndSensorInfo,
              org.registered["activeCountByOS"],
              org.registered["activeCountBySensorVersion"],
              org.registered["activeCountByOSAndSensorVersion"]));

          osBucket.activeSensorDataRecent.countByOSAndSensorVersion.buckets.forEach(osAndSensorInfo =>
            populateAllCountBy(
              osAndSensorInfo,
              org.registered["activeCountByOSRecent"],
              org.registered["activeCountBySensorVersionRecent"],
              org.registered["activeCountByOSAndSensorVersionRecent"]));
      }

      function populateDeploymentTypeFields(sensorInfo) {
        sensorInfo.activeSensorData.countByDeploymentType.buckets.forEach(deploymentTypeSensorInfo =>
          populateDeploymentType(
            deploymentTypeSensorInfo,
            deploymentTypeSensorInfo.countBySubDeploymentType.buckets,
            org.registered["activeCountByDeploymentType"]));

        sensorInfo.activeSensorDataRecent.countByDeploymentType.buckets.forEach(
          deploymentTypeSensorInfo => populateDeploymentType(
            deploymentTypeSensorInfo,
            deploymentTypeSensorInfo.countBySubDeploymentType.buckets,
            org.registered["activeCountByDeploymentTypeRecent"]));

        var subDeploymentTypeBuckets = sensorInfo.activeSensorData.countByDeploymentType.buckets
          .flatMap(deploymentTypeSensorInfo => deploymentTypeSensorInfo.countBySubDeploymentType.buckets);

        populateGoldenCount(subDeploymentTypeBuckets, org.registered["activeGoldenDevices"]);

        var subDeploymentTypeBucketsRecent = sensorInfo.activeSensorDataRecent.countByDeploymentType.buckets
          .flatMap(deploymentTypeSensorInfo => deploymentTypeSensorInfo.countBySubDeploymentType.buckets);

        populateGoldenCount(subDeploymentTypeBucketsRecent, org.registered["activeGoldenDevicesRecent"]);
      }

      /**
       * The calculation logic is moved in the query, look at the CBC-32295 about more details why.
       * @param {*} protectionHoursByDeploymentTypeLastDay - array with the registered and deregistered values per sensor;
       * @param {*} start_time_last_day - start of the period we are calculating for;
       * @param {*} end_time_last_day - end of the period we are calculating for;
       *
       * --Ca---Cb-Xa---Da-----|-----Ya-------Za-------Xb--------------Yb----------|--------------t1-----------------------|
		                       start_time                                      end_time/start_time                          end_time

            Case X Sensor - Protection Hours between 27th - 28th Feb = (Xb - start_time)
            Case Y Sensor - Protection Hours between 27th - 28th Feb = (Yb - Ya)
            Case Z Sensor - Protection Hours between 27th - 28th Feb = (end_time - Za)
            Case D Sensor - Protection Hours between 27th - 28th Feb = (end_time - start_time) (24 hours)
            Case C Sensor - Protection Hours between 27th - 28th Feb - skipped because it was calculated in the previous day

            We are calculating the protection hours of the sensors for a 24h timeframe. Above are the five different use cases we
            have to cover, where 'a' values shows the time device was registered, while the 'b' when it was deregistered.
       *
       */
      function populateConsumptionMetricsExport(protectionHoursByDeploymentTypeLastDay, start_time_last_day, end_time_last_day) {
        protectionHoursByDeploymentTypeLastDay.buckets.forEach(function (protectionHoursEntry) {
          org.protectionHoursByDeploymentTypeLastDay.push({"key": protectionHoursEntry.key, "protectionHours": protectionHoursEntry.protectionTime.value});
          org.registeredCountByDeploymentTypeLastDay.push({"key": protectionHoursEntry.key, "count": protectionHoursEntry.registeredCountLastDay.doc_count});
        });
      }

      if(!_.isUndefined(_.first(orgIdEntry.orgs.buckets)) && !_.isUndefined(_.first(orgIdEntry.orgs.buckets).key))
        org.name = _.first(orgIdEntry.orgs.buckets).key;
      else
        org.name = "unknown";

      orgIdEntry.status.buckets.forEach(function (statusEntry) {
                    switch(statusEntry.key) {
                        case 'REGISTERED':
                                org.registered.total = statusEntry.doc_count;
                                var windows = _.findWhere(statusEntry.deviceType.buckets, {"key" : "WINDOWS"});
                                if (windows) {
                                  populateCountFields(windows, "Windows");
                                }
                                var mac = _.findWhere(statusEntry.deviceType.buckets, {"key" : "MAC"});
                                if (mac) {
                                  populateCountFields(mac, "Mac");
                                }
                                var linux = _.findWhere(statusEntry.deviceType.buckets, {"key": "LINUX"});
                                if(linux) {
                                  populateCountFields(linux, "Linux");
                                }
                                org.registered.totalActive = org.registered.activeMac + org.registered.activeWindows + org.registered.activeLinux;
                                org.registered.totalActiveRecent = org.registered.activeWindowsRecent + org.registered.activeMacRecent + org.registered.activeLinuxRecent;
                                var totalInactiveRecent = org.registered.inactiveWindowsRecent + org.registered.inactiveMacRecent + org.registered.inactiveLinuxRecent;
                                org.registered.totalRecent = org.registered.totalActiveRecent + totalInactiveRecent;

                                statusEntry.deviceType.buckets.forEach(deviceTypeEntry => populateDeploymentTypeFields(deviceTypeEntry));

                                break;
                        case 'QUARANTINE':
                                org.quarantine = statusEntry.doc_count;
                                break;
                        case 'DEREGISTERED':
                                org.deregistered = statusEntry.doc_count;
                                break;
                        case 'DELETED':
                                org.deleted = statusEntry.doc_count;
                                break;
                        case 'BYPASS':
                                org.bypass = statusEntry.doc_count;

                                org.activeBypass.totalActive = statusEntry.deviceType.buckets
                                .flatMap(activeDevicesByOs => activeDevicesByOs.activeDevices.doc_count)
                                .reduce((a, b) => a + b, 0);

                                org.activeBypass.totalActiveRecent = statusEntry.deviceType.buckets
                                .flatMap(activeDevicesByOSRecent => activeDevicesByOSRecent.activeDevicesRecent.doc_count)
                                .reduce((a, b) => a + b, 0);

                                org.bypassTotalConsumedLastDay = statusEntry.bypassTotalConsumedLastDay.doc_count;
                                break;
                        case 'PENDING':
                                org.pending = statusEntry.doc_count;
                                break;
                        default:
                                console.log('invalid status: ' + statusEntry.key);
                    }
            });

            org.totalConsumedLastDay = orgIdEntry.totalConsumedLastDay.doc_count;

            orgIdEntry.totalConsumedLastDay.totalConsumedByDeploymentTypeLastDay.buckets.forEach(
              deploymentTypeSensorInfo => populateDeploymentType(
                deploymentTypeSensorInfo,
                deploymentTypeSensorInfo.totalConsumedLastDayBySubDeploymentType.buckets,
                org["totalConsumedByDeploymentTypeLastDay"]));

            orgIdEntry.totalConsumedLastDay.countByOSAndSensorVersion.buckets.forEach(osAndSensorInfo =>
              populateAllCountBy(
                osAndSensorInfo,
                org["totalConsumedByOsLastDay"],
                org["totalConsumedBySensorVersionLastDay"],
                org["totalConsumedByOsAndSensorVersionLastDay"]));

            var subDeploymentTypeBuckets = orgIdEntry.totalConsumedLastDay.totalConsumedByDeploymentTypeLastDay.buckets
              .flatMap(deploymentTypeInfo => deploymentTypeInfo.totalConsumedLastDayBySubDeploymentType.buckets);

            populateGoldenCount(subDeploymentTypeBuckets, org["totalConsumedGoldenDevicesLastDay"]);

            populateConsumptionMetricsExport(orgIdEntry.protectionHoursByDeploymentTypeLastDay, last_day_start_time, last_day_end_time);

            console.log(JSON.stringify(org));
      });
  }
};

async function dailyExport(client, is_mcs2) {
  // find max orgId
  await client.search({
      ...(!is_mcs2 && {requestTimeout: 30000}),
      index: 'a_devicestats',
      type: 'device',
      body: {
          "query": {
              "query_string": {
                  "query": "*"
              }
          },
          "aggregations" : {
              "max_orgId" : {
                  "max" : {
                      "field" : "organizationId"
                  }
              }
          },
          "size": 0
      }
  }).then(
      function (response) {
          agg = getResponse(response, is_mcs2);

          if (!_.isUndefined(agg.aggregations.max_orgId)) {
              // console.log("max orgid: " + agg.aggregations.max_orgId.value);
              max_org_id = agg.aggregations.max_orgId.value;
          } else {
              console.log("unable to fetch max org_id to iterate from devicestats");
              process.exit(1);
          }
      }, function (error){
          console.log(error.message);
          process.exit(1);
      });

  // console.log("start processing");
  // get org stats in batches
  var from_org_id = 0;
  var maxRetries = 5;
  var retry = 0;
  do {
      to_org_id = from_org_id + batch;
      var queryToExecute = getQuery(from_org_id, to_org_id, is_mcs2);
      await client.search(queryToExecute)
                  .then(
                      function (response) {
                          //console.log("processing response.body orgs (" + from_org_id + " .. " + to_org_id + ")");
                          processResponse(response, is_mcs2);
                          from_org_id = to_org_id;
                          retry = 0;
                      }, function (error){
                          retry = retry + 1;
                          if (retry > maxRetries) {
                              console.log('failed after ('+ maxRetries + ') retries. ' + error.message);
                              process.exit(1);
                          }
                      });
  } while (to_org_id < max_org_id);
  //console.log("done processing");
  process.exit(0);
}

(async() => {
  if(es_port == 443) {
    var is_mcs2 = true;
  } else {
    var is_mcs2 = false;
  }

  checkParams(
    es_host,
    es_port,
    es_user,
    es_password,
    is_mcs2,
    "app-orgtats.js",
    is_mcs2 ?
    "<es_host> <es_port> <es_user> <es_password>  <cbd_env> <optional: start_time> <optional: recent_start_time>" :
    "<es_host> <es_port> <cbd_env> <optional: start_time> <optional: recent_start_time>");

  var client = getClient(es_host, es_port, es_user, es_password, is_mcs2)

  if(!is_mcs2) {
    client.ping({
      requestTimeout: 30000,
    }, function (error) {
      if (error) {
        console.error('The cluster is down!');
        console.error(error);
        process.exit(1);
      }
    });
  }

  await dailyExport(client, is_mcs2);
})()
